<?php
use YooKassa\Client;
use YooKassa\Model\Notification\NotificationFactory;
use YooKassa\Model\PaymentStatus;
use YooKassa\Request\Payments\Payment\CreateCaptureRequest;
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentYookassa_settings_mapper
{
    public function success()
    {
        header("Content-type: text/xml; charset=utf-8");

        $body = @file_get_contents('php://input');
        $callbackParams = json_decode($body, true);
        if (json_last_error()) {
            header("HTTP/1.1 400 Bad Request");
            header("Status: 400 Bad Request");
            return;
        }

        $notificationFactory = new NotificationFactory();
        $notificationModel = $notificationFactory->factory($callbackParams);

        $paymentInfo = $notificationModel->getObject();
        $paymentId = $paymentInfo->getId();
        $orderId = (int) $paymentInfo->getMetadata()->offsetGet('order_id');

        if (!$orderId) {
            header("HTTP/1.1 400 Bad Request");
            header("Status: 400 Bad Request");
            return;
        }

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            header("HTTP/1.1 200 OK");
            header("Status: 200 OK");
            return;
        }

        if (!$ord2->payment()->id() || $ord2->payment()->status() == 'off')
        {
            header("HTTP/1.1 400 Bad Request");
            header("Status: 400 Bad Request");
            return;
        }

        includeLangConst($ord2->langId());

        $client = new Client();
        $client->setAuth($ord2->payment()->settings('shopId'), $ord2->payment()->settings('secretKey'));
        $paymentInfo = $client->getPaymentInfo($paymentId);
        if (empty($paymentInfo)) {
            header("HTTP/1.1 404 Not Found");
            header("Status: 404 Not Found");
            return;
        }

        switch ($paymentInfo->status) {
            case PaymentStatus::WAITING_FOR_CAPTURE:
                $captureResult = $this->capturePayment($client, $paymentInfo);
                if ($captureResult->status === PaymentStatus::SUCCEEDED)
                {
                    $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
                    $orderEvent->run('paid');
                }
                header("HTTP/1.1 200 OK");
                header("Status: 200 OK");
                break;
            case PaymentStatus::SUCCEEDED:
                $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
                $orderEvent->run('paid');

                header("HTTP/1.1 200 OK");
                header("Status: 200 OK");
                break;
            case PaymentStatus::PENDING:
                header("HTTP/1.1 400 Bad Request");
                header("Status: 400 Bad Request");
                break;
            case PaymentStatus::CANCELED:
                header("HTTP/1.1 200 OK");
                header("Status: 200 OK");
                break;
        }
    }

    protected function capturePayment($client, $paymentInfo)
    {
        $captureRequest = CreateCaptureRequest::builder()->setAmount($paymentInfo->getAmount())->build();

        $result = $client->capturePayment(
            $captureRequest,
            $paymentInfo->id
        );

        return $result;
    }

	public function getPaymentUrl(Order $ord2)
    {
        $IDN = new \idna_convert(array('idn_version'=>2008));

        $receipt = [];
        if ($ord2->payment()->settings('sendCheck'))
        {
        	$quantity = (float) $ord2->productQuantity();
            $items = [];
            $items[0] = [
                'quantity' => $quantity,
                'description' => $ord2->productName(),
                'amount' => array('value' => round($ord2->price() / $quantity, 2),
                                  "currency" => "RUB"),
                'vat_code' => $ord2->payment()->settings('tax'),
            ];

            if ($ord2->payment()->settings('sno'))
            {
                $items[0]['tax_system_code'] = $ord2->payment()->settings('sno');
            }
            if ($ord2->payment()->settings('paymentSubjectType'))
            {
                $items[0]['payment_subject'] = $ord2->payment()->settings('paymentSubjectType');
            }
            if ($ord2->payment()->settings('paymentMethodType'))
            {
                $items[0]['payment_mode'] = $ord2->payment()->settings('paymentMethodType');
            }

            $customer = [];
            if ($ord2->customer()->email()) {
                $customer["email"] = $IDN->encode($ord2->customer()->email());
            }
            if ($ord2->customer()->phone()) {
                $customer["phone"] = $ord2->customer()->phone();
            }

            $receipt = [
                "customer" => $customer,
                "items" => $items
            ];

        }

        try {

            $client = new Client();
            $client->setAuth($ord2->payment()->settings('shopId'), $ord2->payment()->settings('secretKey'));

            $data = [
                'amount' => array(
                    'value' => $ord2->price(),
                    'currency' => 'RUB',
                ),
                'confirmation' => array(
                    'type' => 'redirect',
                    'return_url' => $ord2->payment()->settings('successURL') ? $ord2->payment()->settings('successURL') : siteProtocol.'://'.$IDN->decode(DOMAIN).BASE_DIR,
                ),
                'capture' => true,
                'description' => $ord2->name(),
                'metadata' => array(
                    'order_id' => $ord2->id(),
                )
            ];

            if (!empty($receipt)) {
                $data['receipt'] = $receipt;
            }

            $payment = $client->createPayment(
                $data,
                uniqid('', true)
            );

            return $payment->getConfirmation()->getConfirmationUrl();
        }
        catch (\Throwable $e)
        {
            throw new NotFoundException($e->getMessage());
        }

    }

    public function settings($p)
    {
        if (!isset($p['settings']['shopId']))
        {
            $p['settings']['shopId'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        if (!isset($p['settings']['tax']))
        {
            $p['settings']['tax'] = '';
        }

        if (!isset($p['settings']['paymentSubjectType']))
        {
            $p['settings']['paymentSubjectType'] = '';
        }

        if (!isset($p['settings']['paymentMethodType']))
        {
            $p['settings']['paymentMethodType'] = '';
        }

        if (!isset($p['settings']['sno']))
        {
            $p['settings']['sno'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}